<?php

namespace DHLParcel\Shipping\Model\Core;

use AppKernel;
use DHLParcel\Shipping\Model\Twig\DHLParcelExtension;
use ReflectionMethod;
use Twig_Environment;
use Configuration;

class Kernel extends SingletonAbstract
{
    /** @var AppKernel */
    public $kernel;
    /** @var Twig_Environment */
    private $twig;
    private $configuration;

    public static function instance()
    {
        $instance = parent::instance();
        $instance->init();
        return $instance;
    }

    protected function init()
    {
        if (!$this->kernel) {
            global $kernel;

            if (isset($kernel)) {
                $this->kernel = $kernel;
            } else {
                require_once _PS_ROOT_DIR_ . '/app/AppKernel.php';
                $env = _PS_MODE_DEV_ ? 'dev' : 'prod';
                $debug = _PS_MODE_DEV_ ? true : false;
                $this->kernel = $kernel = new AppKernel($env, $debug);
                $this->kernel->boot();
            }
        }
    }

    /**
     * @return object|Twig_Environment
     * adding a singular twig
     */
    protected function getTwig()
    {
        if (!$this->twig) {
            $this->twig = $this->kernel->getContainer()->get('twig');
            $this->addTwigFunctions($this->twig);
        }
        return $this->twig;
    }

    /**
     * @param Twig_Environment $twig
     * adds extra function to fix pre 1.7.5 installations to allow dynamic importing
     * 1.7.4 doesn't have @Modules
     *
     * in future tense are more twig functions are needed they should be moved to a twig extension instead
     * view link bellow for more info
     * https://twig.symfony.com/doc/1.x/advanced.html#id2
     */
    protected function addTwigFunctions($twig)
    {
        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            return;
        }

        $function = new \Twig\TwigFunction('addBase', function ($path) {
            if (version_compare(_PS_VERSION_, '1.7.5', '>=')) {
                $template = '@Modules/dhlparcel_shipping/views/templates/' . $path;
            } elseif (version_compare(_PS_VERSION_, '1.7.0', '>=') && version_compare(_PS_VERSION_, '1.7.5', '<')) {
                $template = _PS_MODULE_DIR_ . 'dhlparcel_shipping/views/templates/' . $path;
            } else {
                $template = '';
            }
            return $template;
        });
        $twig->addFunction($function);

        $function = new \Twig\TwigFunction('translate', function ($string) {
            return \Module::getInstanceByName('dhlparcel_shipping')->l($string, 'kernel');
        });
        $twig->addFunction($function);
    }

    public function render($path, $data = [])
    {
        if (!$this->kernel) {
            return '';
        }
        if (version_compare(_PS_VERSION_, '1.7.5', '>=')) {
            $template = '@Modules/dhlparcel_shipping/views/templates/' . $path . '.twig';
        } elseif (version_compare(_PS_VERSION_, '1.7.0', '>=') && version_compare(_PS_VERSION_, '1.7.5', '<')) {
            $template = _PS_MODULE_DIR_ . 'dhlparcel_shipping/views/templates/' . $path . '.twig';
        } else {
            return false;
        }

        return $this->getTwig()->render($template, $data);
    }

    protected function templateStrings()
    {
        return [
            // backoffice\carrier\form
            $this->l('No'),
            $this->l('Yes'),
            $this->l('Link with DHL services'),
            // backoffice\order\panel
            $this->l('Shipment type'),
            $this->l('Delivery method'),
            $this->l('Service options'),
            $this->l('Size and weight'),
            $this->l('Create label'),
            // backoffice\order\panel\*
            $this->l('Consumer'),
            $this->l('Business'),
            $this->l('Not available'),
            $this->l('Size'),
            $this->l('Services'),
            $this->l('Trackercode'),
            $this->l('Actions'),
            // backoffice\order\panel\input\*
            $this->l('Change'),
            $this->l('Select a DHL ServicePoint'),
            $this->l('Search zipcode, city or name'),
            $this->l('No DHL ServicePoint found'),
            // backoffice\settings
            $this->l('Test connection and retrieve account data'),
            $this->l('Open PrestaShop settings'),
            $this->l('Manage'),
            $this->l('DHL shipping methods can be managed in the default PrestaShop shipping settings.'),
            $this->l('Reinstall DHL shipping methods'),
            $this->l('Reset'),
            $this->l('Reinstall default DHL shipping methods to PrestaShop for in case you accidentally removed them. The shipping method values will reset to default values.'),
            $this->l('Link payment methods'),
            $this->l('Enable payment methods for DHL shipping methods in PrestaShop payment settings.'),
            // backoffice\settings\bulk
            $this->l('Add'),
            $this->l('Bulk Profiles'),
            $this->l('Select or create a profile.'),
            $this->l('Available services based on receiver country'),
            $this->l('Show available services based on country. This setting will not change the address of the receiver or limit on what countries you can use this profile for.'),
            //$this->l('Shipment type'),
            $this->l('Default'),
            $this->l('Send to private address'),
            $this->l('Send to business address'),
            $this->l('Delivery type'),
            $this->l('Service Options'),
            $this->l('If required options are unavailable, skip creating label'),
            // $this->l('Size'),
            $this->l('Please setup your connection and shipping address first in order to setup bulk actions.'),
            $this->l('selected'),
            $this->l('In the mailbox (except for ServicePoint orders)'),
            $this->l('Enable'),
            $this->l('Required'),
            $this->l('Automatically select smallest available'),
            $this->l('Auotmatically select largest available'),
            // frontoffice\checkout
            $this->l('You have selected'),
            $this->l('Choose a different DHL ServicePoint'),
            $this->l('Please choose a DHL ServicePoint.'),
            $this->l('Please choose a DHL ServicePoint'),
        ];
    }

    // phpcs:ignore
    protected function l($string = null)
    {
        // This function does nothing, it's only purpose is to allow PrestaShop to scan for translations
        return null;
    }

    // Legacy configuration, for multistore
    // See: https://devdocs.prestashop.com/1.7/development/multistore/shops-configuration/
    public function getMultistoreConfiguration($configString, $shopId = null)
    {
        if (!empty($shopId)
            && version_compare(_PS_VERSION_, '1.7.7', '>=')
            && class_exists('\PrestaShop\PrestaShop\Core\Domain\Shop\ValueObject\ShopConstraint')
        ) {
            if (method_exists(\PrestaShop\PrestaShop\Core\Domain\Shop\ValueObject\ShopConstraint::class, '__construct')) {
                $reflection = new ReflectionMethod(\PrestaShop\PrestaShop\Core\Domain\Shop\ValueObject\ShopConstraint::class, '__construct');
                if ($reflection->isPublic()) {
                    $shopConstraint = new \PrestaShop\PrestaShop\Core\Domain\Shop\ValueObject\ShopConstraint($shopId, null);
                    return $this->getConfiguration()->get($configString, null, $shopConstraint);
                }
            }
        }
        return Configuration::get($configString);
    }

    protected function getConfiguration()
    {
        if (!$this->configuration) {
            $this->configuration = $this->kernel->getContainer()->get('prestashop.adapter.legacy.configuration');
        }
        return $this->configuration;
    }

    public function isMultistoreActive()
    {
        // Not exactly when multistore check is available, but we tested it with 1.7.6.3 and the earlier versions don't have this service
        if (version_compare(_PS_VERSION_, '1.7.6.3', '<')) {
            return false;
        }

        // Get the multistore feature adapter from the service container
        $multistoreFeature = $this->kernel->getContainer()->get('prestashop.adapter.feature.multistore');

        // Is the multistore feature enabled?
        $isMultistoreEnabled = $multistoreFeature->isActive();

        // This will check that multistore is enabled AND that there are at least two active shops
        $isMultistoreUsed = $multistoreFeature->isUsed();

        return $isMultistoreEnabled && $isMultistoreUsed;
    }
}
